@@ -14,27 +14,31 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
-#include <time.h>
 #include <dirent.h>
 #include <pthread.h>
 #include <sys/wait.h>
 #include "global.h"
 #include "client.h"
+#ifdef HAVE_COMMAND
 #include "command.h"
+#endif
+#ifdef HAVE_MONITOR
+#include "monitor.h"
+#endif
 #include "log.h"
 
-#define MAX_PASSWORD_DELAY_TIMER 60
+#define MAX_PASSWORD_DELAY_TIMER (int)(MINUTE / TASK_RUNNER_INTERVAL)
 
 typedef struct type_client {
 	t_session *session;
-	int delayed_remove;
+	time_t remove_deadline;
 
 	struct type_client *next;
 } t_client;
 
 typedef struct type_banned {
 	t_ip_addr ip;
-	int       timer;
+	time_t    deadline;
 	int       bantime;
 	unsigned long connect_attempts;
 	
@@ -47,7 +51,7 @@ static t_banned *banlist;
 static pthread_mutex_t ban_mutex;
 static t_ipcounterlist *wrong_password_list;
 static pthread_mutex_t pwd_mutex;
-static int delay_timer = 0;
+static int password_delay_timer = 0;
 
 /* Initialize this module.
  */
@@ -72,7 +76,7 @@ int add_client(t_session *session) {
 
 	if ((new = (t_client*)malloc(sizeof(t_client))) != NULL) {
 		new->session = session;
-		new->delayed_remove = TIMER_OFF;
+		new->remove_deadline = TIMER_OFF;
 
 		i = index_by_ip(&(session->ip_address));
 
@@ -104,7 +108,7 @@ int mark_client_for_removal(t_session *session, int delay) {
 	while (list != NULL) {
 		if (list->session == session) {
 			close_socket(list->session);
-			list->delayed_remove = delay - 1;
+			list->remove_deadline = time(NULL) + delay;
 			result = 1;
 			break;
 		}
@@ -116,10 +120,10 @@ int mark_client_for_removal(t_session *session, int delay) {
 	return result;
 }
 
-/* Check the delayed_remove timers and remove client
+/* Check the remove_deadline timers and remove client
  * when timer has reached 0
  */
-void check_delayed_remove_timers(t_config *config) {
+void check_remove_deadlines(t_config *config, time_t now) {
 	t_client *client, *prev = NULL, *next;
 	int i;
 
@@ -133,22 +137,18 @@ void check_delayed_remove_timers(t_config *config) {
 		client = client_list[i];
 		while (client != NULL) {
 			next = client->next;
-			switch (client->delayed_remove) {
-				case TIMER_OFF:
-					prev = client;
-					break;
-				case 0:
-					free(client->session);
-					free(client);
-					if (prev == NULL) {
-						client_list[i] = next;
-					} else {
-						prev->next = next;
-					}
-					break;
-				default:
-					client->delayed_remove--;
-					prev = client;
+			if (client->remove_deadline == TIMER_OFF) {
+				prev = client;
+			} else if (client->remove_deadline <= now) {
+				free(client->session);
+				free(client);
+				if (prev == NULL) {
+					client_list[i] = next;
+				} else {
+					prev->next = next;
+				}
+			} else {
+				prev = client;
 			}
 			client = next;
 		}
@@ -204,38 +204,48 @@ int remove_client(t_session *session, bool free_session) {
 /* Check whether to allow or deny a new connection.
  */
 int connection_allowed(t_ip_addr *ip, int maxperip, int maxtotal) {
-	bool banned = false;
 	t_banned *ban;
 	int perip = 0, total = 0, i;
 	t_client *client;
+	time_t now;
 
+	now = time(NULL);
+
+	/* Check bans
+	 */
 	pthread_mutex_lock(&ban_mutex);
 
 	ban = banlist;
 	while (ban != NULL) {
 		if (same_ip(&(ban->ip), ip)) {
+			/* Ban expired?
+			 */
+			if (now >= ban->deadline) {
+				break;
+			}
+
 			ban->connect_attempts++;
-			banned = true;
-			break;
+			pthread_mutex_unlock(&ban_mutex);
+			return ca_BANNED;
 		}
 		ban = ban->next;
 	}
 
 	pthread_mutex_unlock(&ban_mutex);
 
-	if (banned) {
-		return ca_BANNED;
-	}
-
+	/* Check max connections
+	 */
 	for (i = 0; i < 256; i++) {
 		pthread_mutex_lock(&client_mutex[i]);
 
 		client = client_list[i];
 		while (client != NULL) {
 			if (same_ip(&(client->session->ip_address), ip)) {
-				perip++;
+				if (now < client->remove_deadline) {
+					perip++;
+				}
 			}
-			if (client->delayed_remove == TIMER_OFF) {
+			if (client->remove_deadline == TIMER_OFF) {
 				total++;
 			}
 			client = client->next;
@@ -320,7 +330,7 @@ int kick_ip(t_ip_addr *ip) {
 bool client_is_flooding(t_session *session) {
 	time_t time_passed;
 
-	time_passed = time(NULL) - session->flooding_timer + 1;
+	time_passed = session->time - session->flooding_timer + 1;
 
 	return ((session->kept_alive * session->config->flooding_time) > (session->config->flooding_count * time_passed));
 }
@@ -362,8 +372,8 @@ int ban_ip(t_ip_addr *ip, int timer, bool kick_on_ban) {
 	ban = banlist;
 	while (ban != NULL) {
 		if (same_ip(&(ban->ip), ip)) {
-			ban->timer = timer;
 			ban->bantime = timer;
+			ban->deadline = time(NULL) + ban->bantime;
 			new_ip = false;
 			break;
 		}
@@ -373,8 +383,8 @@ int ban_ip(t_ip_addr *ip, int timer, bool kick_on_ban) {
 	if (new_ip) {
 		if ((ban = (t_banned*)malloc(sizeof(t_banned))) != NULL) {
 			copy_ip(&(ban->ip), ip);
-			ban->timer = timer;
 			ban->bantime = timer;
+			ban->deadline = time(NULL) + ban->bantime;
 			ban->connect_attempts = 0;
 			ban->next = banlist;
 			banlist = ban;
@@ -409,7 +419,7 @@ int reban_ip(t_ip_addr *ip) {
 	ban = banlist;
 	while (ban != NULL) {
 		if (same_ip(&(ban->ip), ip)) {
-			ban->timer = ban->bantime;
+			ban->deadline = time(NULL) + ban->bantime;
 			result = 1;
 			break;
 		}
@@ -423,7 +433,7 @@ int reban_ip(t_ip_addr *ip) {
 
 /* Check the timers of the banlist.
  */
-void check_ban_list(t_config *config) {
+void check_ban_list(t_config *config, time_t now) {
 	t_banned *ban, *prev = NULL, *next;
 
 	pthread_mutex_lock(&ban_mutex);
@@ -431,22 +441,24 @@ void check_ban_list(t_config *config) {
 	ban = banlist;
 	while (ban != NULL) {
 		next = ban->next;
-		switch (ban->timer) {
-			case TIMER_OFF:
-				prev = ban;
-				break;
-			case 0:
-				if (prev == NULL) {
-					banlist = next;
-				} else {
-					prev->next = next;
-				}
-				log_unban(config->system_logfile, &(ban->ip), ban->connect_attempts);
-				free(ban);
-				break;
-			default:
-				ban->timer--;
-				prev = ban;
+		if (ban->deadline == TIMER_OFF) {
+			/* Timer off
+			 */
+			prev = ban;
+		} else if (ban->deadline <= now) {
+			/* Deadline reached
+			 */
+			if (prev == NULL) {
+				banlist = next;
+			} else {
+				prev->next = next;
+			}
+			log_unban(config->system_logfile, &(ban->ip), ban->connect_attempts);
+			free(ban);
+		} else {
+			/* other
+			 */
+			prev = ban;
 		}
 		ban = next;
 	}
@@ -520,6 +532,11 @@ int register_wrong_password(t_session *session) {
 					ban_ip(&(session->ip_address), session->config->ban_on_wrong_password, session->config->kick_on_ban);
 					session->keep_alive = false;
 					log_system(session, "Client banned because of too many wrong passwords");
+#ifdef HAVE_MONITOR
+					if (session->config->monitor_enabled) {
+						monitor_counter_ban(session);
+					}
+#endif
 				}
 			}
 
@@ -551,7 +568,7 @@ void remove_wrong_password_list(t_config *config) {
 
 	if ((config->ban_on_wrong_password == 0) || (wrong_password_list == NULL)) {
 		return;
-	} else if (++delay_timer < MAX_PASSWORD_DELAY_TIMER) {
+	} else if (++password_delay_timer < MAX_PASSWORD_DELAY_TIMER) {
 		return;
 	}
 
@@ -569,7 +586,7 @@ void remove_wrong_password_list(t_config *config) {
 		free(remove);
 	}
 
-	delay_timer = 0;
+	password_delay_timer = 0;
 }
 
 /* Close all client sockets to run a CGI program
@@ -594,23 +611,28 @@ void print_client_list(FILE *fp) {
 	t_client *client;
 	char ip_address[MAX_IP_STR_LEN];
 	int i, count = 0;
+	time_t now;
+
+	now = time(NULL);
 
 	for (i = 0; i < 256; i++) {
 		pthread_mutex_lock(&client_mutex[i]);
 
 		client = client_list[i];
 		while (client != NULL) {
-			fprintf(fp, "  Client ID   : %d\n", client->session->client_id);
-			if (inet_ntop(client->session->ip_address.family, &(client->session->ip_address.value), ip_address, MAX_IP_STR_LEN) != NULL) {
-				fprintf(fp, "  IP-address  : %s\n", ip_address);
+			if ((client->remove_deadline == TIMER_OFF) || (now < client->remove_deadline)) {
+				fprintf(fp, "  Client ID   : %d\n", client->session->client_id);
+				if (inet_ntop(client->session->ip_address.family, &(client->session->ip_address.value), ip_address, MAX_IP_STR_LEN) != NULL) {
+					fprintf(fp, "  IP-address  : %s\n", ip_address);
+				}
+				fprintf(fp, "  Socket      : %d\n", client->session->client_socket);
+				if (client->session->remote_user != NULL) {
+					fprintf(fp, "  Remote user : %s\n", client->session->remote_user);
+				}
+				fprintf(fp, "  Kept alive  : %d\n\n", client->session->kept_alive);
+				count++;
 			}
-			fprintf(fp, "  Socket      : %d\n", client->session->client_socket);
-			if (client->session->remote_user != NULL) {
-				fprintf(fp, "  Remote user : %s\n", client->session->remote_user);
-			}
-			fprintf(fp, "  Kept alive  : %d\n\n", client->session->kept_alive);
 			client = client->next;
-			count++;
 		}
 
 		pthread_mutex_unlock(&client_mutex[i]);
@@ -644,17 +666,22 @@ void print_ban_list(FILE *fp) {
 	t_banned *ban;
 	char ip_address[MAX_IP_STR_LEN];
 	int count = 0;
+	time_t now;
+
+	now = time(NULL);
 
 	pthread_mutex_lock(&ban_mutex);
 
 	ban = banlist;
 	while (ban != NULL) {
-		if (inet_ntop(ban->ip.family, &(ban->ip.value), ip_address, MAX_IP_STR_LEN) != NULL) {
-			fprintf(fp, "  IP-address  : %s\n", ip_address);
+		if (now < ban->deadline) {
+			if (inet_ntop(ban->ip.family, &(ban->ip.value), ip_address, MAX_IP_STR_LEN) != NULL) {
+				fprintf(fp, "  IP-address  : %s\n", ip_address);
+			}
+			fprintf(fp, "  seconds left: %ld\n\n", (long)ban->deadline);
+			count++;
 		}
-		fprintf(fp, "  seconds left: %d\n\n", ban->timer);
 		ban = ban->next;
-		count++;
 	}
 	fprintf(fp, "  Total: %d bans\n", count);
 
