@@ -17,6 +17,7 @@
 #include <unistd.h>
 #include <string.h>
 #include <fcntl.h>
+#include <time.h>
 #include <signal.h>
 #include <dirent.h>
 #include <errno.h>
@@ -64,10 +65,11 @@
 #ifdef HAVE_XSLT
 #include "xslt.h"
 #endif
+#ifdef HAVE_MONITOR
+#include "monitor.h"
+#endif
 
 #define REQUEST_BUFFER_CHUNK     4 * KILOBYTE
-#define MAIN_LOOP_TIMER          1
-#define PTHREAD_STACK_SIZE     512 * KILOBYTE
 
 #define fe_MAX_REQUESTSIZE      -2
 #define fe_TIMEOUT              -3
@@ -91,6 +93,7 @@ typedef struct {
 
 volatile int received_signal = rs_NONE;
 bool               must_quit = false;
+int open_connections = 0;
 
 char *hs_conlen      = "Content-Length: ";
 char *fb_filesystem  = "access denied via filesystem";
@@ -99,25 +102,31 @@ char *fb_accesslist  = "access denied via accesslist";
 char *fb_alterlist   = "access denied via alterlist";
 char *version_string = "Hiawatha v"VERSION
 #ifdef HAVE_CACHE
-			", cache"
+	", cache"
+#endif
+#ifdef HAVE_CHROOT
+	", chroot"
 #endif
 #ifdef HAVE_COMMAND
-			", CommandChannel"
+	", CommandChannel"
 #endif
 #ifdef DEBUG
-			", debug"
+	", debug"
 #endif
 #ifdef HAVE_IPV6
-			", IPv6"
+	", IPv6"
+#endif
+#ifdef HAVE_MONITOR
+	", Monitor"
 #endif
 #ifdef HAVE_SSL
-			", SSL"
+	", SSL"
 #endif
 #ifdef HAVE_TOOLKIT
-			", URL toolkit"
+	", URL toolkit"
 #endif
 #ifdef HAVE_XSLT
-			", XSLT"
+	", XSLT"
 #endif
 ;
 
@@ -356,9 +365,9 @@ int fetch_request(t_session *session) {
 	int upload_handle = -1;
 
 	if (session->kept_alive == 0) {
-		deadline = time(NULL) + session->binding->time_for_1st_request;
+		deadline = session->time + session->binding->time_for_1st_request;
 	} else {
-		deadline = time(NULL) + session->binding->time_for_request;
+		deadline = session->time + session->binding->time_for_request;
 	}
 
 	do {
@@ -630,7 +639,7 @@ int parse_request(t_session *session, int total_bytes) {
 		session->body = session->request + session->header_length;
 	}
 	session->headerfields = parse_headerfields(str_end + 3);
-	session->hostname = get_headerfield("Host:", session->headerfields);
+	session->hostname = strlower(get_headerfield("Host:", session->headerfields));
 	session->cookie = get_headerfield("Cookie:", session->headerfields);
 
 	if ((conn = get_headerfield("Connection:", session->headerfields)) != NULL) {
@@ -837,6 +846,8 @@ int serve_client(t_session *session) {
 		return result;
 	}
 
+	session->time = time(NULL);
+
 	/* Hide reverse proxies
 	 */
 	if (in_iplist(session->config->hide_proxy, &(session->ip_address))) {
@@ -877,7 +888,7 @@ int serve_client(t_session *session) {
 			session->host = host_record;
 		}
 	}
-	session->host->access_time = time(NULL);
+	session->host->access_time = session->time;
 
 	if (session->host->secure_url) {
 		if (strstr(session->request_uri, "%00") != NULL) {
@@ -897,10 +908,18 @@ int serve_client(t_session *session) {
 				if ((session->config->ban_on_denied_body > 0) && ip_allowed(&(session->ip_address), session->config->banlist_mask)) {
 					ban_ip(&(session->ip_address), session->config->ban_on_denied_body, session->config->kick_on_ban);
 					log_system(session, "Client banned because of denied body");
+#ifdef HAVE_MONITOR
+					if (session->config->monitor_enabled) {
+						monitor_counter_ban(session);
+					}
+#endif
 				}
 
 				log_string(session->host->error_logfile, "Request denied because of unallowed body content");
 				log_exploit_attempt(session, "denied body", session->body);
+#ifdef HAVE_MONITOR
+				monitor_counter_exploit_attempt(session);
+#endif
 
 				*(session->body + session->content_length) = chr;
 
@@ -926,6 +945,11 @@ int serve_client(t_session *session) {
 			if ((options.ban > 0) && ip_allowed(&(session->ip_address), session->config->banlist_mask)) {
 				ban_ip(&(session->ip_address), options.ban, session->config->kick_on_ban);
 				log_system(session, "Client banned because of URL match in UrlToolkit rule");
+#ifdef HAVE_MONITOR
+				if (session->config->monitor_enabled) {
+					monitor_counter_ban(session);
+				}
+#endif
 				return 403;
 			}
 
@@ -1083,7 +1107,7 @@ int serve_client(t_session *session) {
 		case HEAD:
 			if (session->cgi_type != no_cgi) {
 				session->body = NULL;
-				result = execute_cgi(session);
+				result = execute_cgi(session);	
 #ifdef HAVE_XSLT
 			} else if (can_transform_with_xslt(session)) {
 				result = handle_xml_file(session);
@@ -1092,9 +1116,17 @@ int serve_client(t_session *session) {
 				result = send_file(session);
 			}
 			if (result == 404) {
-				if (session->host->show_index && (session->uri[session->uri_len - 1] == '/')) {
+#ifdef HAVE_XSLT
+				if ((session->host->show_index != NULL) && (session->uri[session->uri_len - 1] == '/')) {
 					result = show_index(session);
 				}
+#endif
+#ifdef HAVE_MONITOR
+			} else if (session->config->monitor_enabled) {
+				if ((result == 200) && (session->host->monitor_host)) {
+					unlink(session->file_on_disk);
+				}
+#endif
 			}
 
 			if ((session->request_method == GET) && (session->cgi_type == no_cgi) && (session->directory != NULL)) {
@@ -1142,18 +1174,30 @@ void handle_request_result(t_session *session, int result) {
 	switch (result) {
 		case fe_MAX_REQUESTSIZE:
 			log_system(session, "Maximum request size reached");
-			send_code(session, 413);
+			session->return_code = 413;
+			send_code(session);
 			if ((session->config->ban_on_max_request_size > 0) && ip_allowed(&(session->ip_address), session->config->banlist_mask)) {
 				ban_ip(&(session->ip_address), session->config->ban_on_max_request_size, session->config->kick_on_ban);
 				log_system(session, "Client banned because of sending a too large request");
+#ifdef HAVE_MONITOR
+				if (session->config->monitor_enabled) {
+					monitor_counter_ban(session);
+				}
+#endif
 			}
 			break;
 		case fe_TIMEOUT:
 			if (session->kept_alive == 0) {
-				send_code(session, 408);
+				session->return_code = 408;
+				send_code(session);
 				if ((session->config->ban_on_timeout > 0) && ip_allowed(&(session->ip_address), session->config->banlist_mask)) {
 					ban_ip(&(session->ip_address), session->config->ban_on_timeout, session->config->kick_on_ban);
 					log_system(session, "Client banned because of connection timeout");
+#ifdef HAVE_MONITOR
+					if (session->config->monitor_enabled) {
+						monitor_counter_ban(session);
+					}
+#endif
 				} else {
 					log_system(session, "Timeout while waiting for request");
 				}
@@ -1176,8 +1220,14 @@ void handle_request_result(t_session *session, int result) {
 			if ((session->config->ban_on_sqli > 0) && ip_allowed(&(session->ip_address), session->config->banlist_mask)) {
 				ban_ip(&(session->ip_address), session->config->ban_on_sqli, session->config->kick_on_ban);
 				log_system(session, "Client banned because of SQL injection");
+#ifdef HAVE_MONITOR
+				if (session->config->monitor_enabled) {
+					monitor_counter_ban(session);
+				}
+#endif
 			}
-			send_code(session, 400);
+			session->return_code = 400;
+			send_code(session);
 			log_request(session, 400);
 			break;
 		case 200:
@@ -1187,7 +1237,8 @@ void handle_request_result(t_session *session, int result) {
 		case 304:
 		case 412:
 			if (session->data_sent == false) {
-				send_header(session, result);
+				session->return_code = result;
+				send_header(session);
 				send_buffer(session, "Content-Length: 0\r\n\r\n", 21);
 			}
 			break;
@@ -1195,20 +1246,27 @@ void handle_request_result(t_session *session, int result) {
 		case 413:
 			session->keep_alive = false;
 			if (session->data_sent == false) {
-				send_header(session, result);
+				session->return_code = result;
+				send_header(session);
 				send_buffer(session, "Content-Length: 0\r\n\r\n", 21);
 			}
 			break;
 		case 400:
 			log_garbage(session);
 			if (session->data_sent == false) {
-				if (send_code(session, result) == -1) {
+				session->return_code = 400;
+				if (send_code(session) == -1) {
 					session->keep_alive = false;
 				}
 			}
 			if ((session->config->ban_on_garbage > 0) && ip_allowed(&(session->ip_address), session->config->banlist_mask)) {
 				ban_ip(&(session->ip_address), session->config->ban_on_garbage, session->config->kick_on_ban);
 				log_system(session, "Client banned because of sending garbage");
+#ifdef HAVE_MONITOR
+				if (session->config->monitor_enabled) {
+					monitor_counter_ban(session);
+				}
+#endif
 			}
 			break;
 		case 401:
@@ -1225,7 +1283,8 @@ void handle_request_result(t_session *session, int result) {
 						break;
 					default:
 						if (session->data_sent == false) {
-							if (send_code(session, result) == -1) {
+							session->return_code = result;
+							if (send_code(session) == -1) {
 								session->keep_alive = false;
 							}
 						}
@@ -1236,7 +1295,8 @@ void handle_request_result(t_session *session, int result) {
 			session->keep_alive = false;
 		default:
 			if (session->data_sent == false) {
-				send_code(session, result);
+				session->return_code = result;
+				send_code(session);
 			}
 	}
 
@@ -1250,7 +1310,16 @@ void handle_request_result(t_session *session, int result) {
 /* Handle the connection of a client.
  */
 void connection_handler(t_session *session) {
-	int result;
+	int result, connections;
+
+#ifdef HAVE_MONITOR
+	connections = ++open_connections;
+	if (session->config->monitor_enabled) {
+		if (connections > session->config->monitor_stats.simultaneous_connections) {
+			session->config->monitor_stats.simultaneous_connections = connections;
+		}
+	}
+#endif
 
 #ifdef HAVE_SSL
 	if (session->binding->use_ssl) {
@@ -1268,6 +1337,14 @@ void connection_handler(t_session *session) {
 				send_buffer(session, NULL, 0); /* Flush the output-buffer */
 			}
 
+#ifdef HAVE_MONITOR
+			if (session->config->monitor_enabled) {
+				monitor_counter_request(session);
+				if (session->host->monitor_requests && (result > 0)) {
+					monitor_request(session);
+				}
+			}
+#endif
 			reset_session(session);
 
 			if (session->keep_alive && (session->config->ban_on_flooding > 0)) {
@@ -1276,6 +1353,11 @@ void connection_handler(t_session *session) {
 						ban_ip(&(session->ip_address), session->config->ban_on_flooding, session->config->kick_on_ban);
 						log_system(session, "Client banned because of flooding");
 						session->keep_alive = false;
+#ifdef HAVE_MONITOR
+						if (session->config->monitor_enabled) {
+							monitor_counter_ban(session);
+						}
+#endif
 					}
 				}
 			}
@@ -1287,6 +1369,10 @@ void connection_handler(t_session *session) {
 		close(session->client_socket);
 	}
 
+#ifdef HAVE_MONITOR
+	open_connections--;
+#endif
+
 	if (session->config->reconnect_delay > 0) {
 		mark_client_for_removal(session, session->config->reconnect_delay);
 	} else {
@@ -1301,30 +1387,48 @@ void connection_handler(t_session *session) {
  */
 void task_runner(t_config *config) {
 	t_ip_addr ip_addr;
+	int delay = 0;
+	time_t now;
 
 	do {
 		sleep(1);
 
-		/* Client checks
-		 */
-		check_ban_list(config);
-		check_delayed_remove_timers(config);
-		remove_wrong_password_list(config);
+		if (delay == TASK_RUNNER_INTERVAL) {
+			now = time(NULL);
 
-		/* FastCGI check
-		 */
-		check_load_balancer(config);
+			/* Client checks
+			 */
+			check_ban_list(config, now);
+			check_remove_deadlines(config, now);
+			remove_wrong_password_list(config);
 
-		/* Close idle logfile handles
-		 */
-		close_logfiles(config->first_host, false);
+			/* FastCGI check
+			 */
+			check_load_balancer(config, now);
+
+			/* Close idle logfile handles
+			 */
+			close_logfiles(config->first_host, now);
 
 #ifdef HAVE_CACHE
-		/* Cache check
-		 */
-		check_cache();
+			/* Cache check
+			 */
+			check_cache(now);
 #endif
 
+#ifdef HAVE_MONITOR
+			/* Monitor stats
+			 */
+			if (config->monitor_enabled) {
+				monitor_stats(config, now);
+			}
+#endif
+
+			delay = 0;
+		} else {
+			delay++;
+		}
+
 #ifdef HAVE_COMMAND
 		/* CommandChannel check
 		 */
@@ -1347,7 +1451,7 @@ void task_runner(t_config *config) {
 				received_signal = rs_NONE;
 				break;
 			case rs_UNLOCK_LOGFILES:
-				close_logfiles(config->first_host, true);
+				close_logfiles(config->first_host, 0);
 				received_signal = rs_NONE;
 				break;
 #ifdef HAVE_CACHE
@@ -1623,6 +1727,11 @@ int accept_connection(t_binding *binding, t_config *config) {
 		case ca_TOOMUCH_PERIP:
 			if ((config->ban_on_max_per_ip > 0) && ip_allowed(&(session->ip_address), session->config->banlist_mask)) {
 				ban_ip(&(session->ip_address), config->ban_on_max_per_ip, config->kick_on_ban);
+#ifdef HAVE_MONITOR
+				if (config->monitor_enabled) {
+					monitor_counter_ban(session);
+				}
+#endif
 			}
 			log_system(session, "Maximum number of connections for IP address reached");
 			break;
@@ -1673,46 +1782,37 @@ int run_server(t_settings *settings) {
 	config = default_config();
 	if (chdir(settings->config_dir) == -1) {
 		perror(settings->config_dir);
-		return EXIT_FAILURE;
+		return -1;
 	} else if (settings->config_check) {
 		printf("Using %s\n", settings->config_dir);
 	}
 	if (read_main_configfile("hiawatha.conf", config, settings->config_check) == -1) {
-		return EXIT_FAILURE;
+		return -1;
 	} else if (check_configuration(config) == -1) {
-		return EXIT_FAILURE;
+		return -1;
 	}
 
 	if (read_mimetypes(config->mimetype_config, &(config->mimetype)) == -1) {
 		fprintf(stderr, "Error while reading mimetype configuration.\n");
-		return EXIT_FAILURE;
+		return -1;
 	}
 
 	if (settings->config_check) {
 		printf("Configuration oke.\n");
-		return EXIT_SUCCESS;
+		return 0;
 	}
-	
-	if (config->wait_for_cgi == false) {
-		signal(SIGCHLD, SIG_IGN);
-	}
-	signal(SIGPIPE, SIG_IGN);
-	signal(SIGTSTP, SIG_IGN);
-	signal(SIGABRT, SIG_IGN);
-	signal(SIGQUIT, SIG_IGN);
-	signal(SIGSEGV, SEGV_handler);
 
 	/* Bind Serverports
 	 */
 	if (bind_sockets(config->binding) == -1) {
-		return EXIT_FAILURE;
+		return -1;
 	}
 
 #ifdef HAVE_SSL
 	fill_random_buffer(config, random_buffer, RANDOM_BUFFER_SIZE);
 	if (ssl_init(random_buffer, RANDOM_BUFFER_SIZE) == -1) {
 		perror("ssl initialize");
-		return EXIT_FAILURE;
+		return -1;
 	}
 
 	binding = config->binding;
@@ -1720,7 +1820,7 @@ int run_server(t_settings *settings) {
 		if (binding->use_ssl) {
 			if ((binding->ssl_context = ssl_binding(binding->ssl_key_cert, binding->required_ca, binding->verify_depth, config->dh_file, config->allowed_ciphers)) == NULL) {
 				perror("bind https");
-				return EXIT_FAILURE;
+				return -1;
 			}
 		}
 		binding = binding->next;
@@ -1731,45 +1831,12 @@ int run_server(t_settings *settings) {
 	/* Bind CommandChannels
 	 */
 	if (bind_sockets(config->command_port) == -1) {
-		return EXIT_FAILURE;
+		return -1;
 	}
 #endif
 
-	/* Create the upload directory for PUT requests
+	/* Misc settings
 	 */
-	if (config->create_upload_dir) {
-		if (mkdir(config->upload_directory, 0733) == -1) {
-			if (errno != EEXIST) {
-				fprintf(stderr, "Error while creating UploadDirectory '%s'\n", config->upload_directory);
-				return EXIT_FAILURE;
-			}
-		}
-		if (stat(config->upload_directory, &status) == -1) {
-			perror("stat(UploadDirectory)");
-			return EXIT_FAILURE;
-		}
-
-		access_rights = 01733;
-		if (status.st_uid != 0) {
-			if ((getuid() == 0) || (geteuid() == 0)) {
-				if (chown(config->upload_directory, 0, 0) == -1) {
-					perror("chown(UploadDirectory, 0, 0)");
-					return EXIT_FAILURE;
-				}
-			} else {
-				access_rights = 01333;
-			}
-		}
-
-		if ((status.st_mode & 07777) != access_rights) {
-			if (chmod(config->upload_directory, access_rights) == -1) {
-				fprintf(stderr, "Can't change access permissions of UploadDirectory '%s'.\n", config->upload_directory);
-				return EXIT_FAILURE;
-			}
-		}
-	}
-
-	touch_logfiles(config, config->server_root);
 	tzset();
 	clearenv();
 
@@ -1779,21 +1846,22 @@ int run_server(t_settings *settings) {
 		switch (pid = fork()) {
 			case -1:
 				perror("fork()");
-				return EXIT_FAILURE;
+				return -1;
 			case 0:
 				if (setsid() == -1) {
 					perror("setsid()");
-					return EXIT_FAILURE;
+					return -1;
 				}
 				break;
 			default:
 				log_pid(config, pid);
-				return EXIT_SUCCESS;
+				return 0;
 		}
 	} else {
 		log_pid(config, getpid());
 	}
 
+#ifdef HAVE_CHROOT
 	/* Change server root
 	 */
 	if (config->server_root != NULL) {
@@ -1804,16 +1872,96 @@ int run_server(t_settings *settings) {
 				}
 			}
 			fprintf(stderr, "\nError while changing root to %s!\n", config->server_root);
-			return EXIT_FAILURE;
+			return -1;
 		} while (false);
+
 #ifdef CYGWIN
 	} else if (chdir("/cygdrive/c") == -1) {
 #else
 	} else if (chdir("/") == -1) {
 #endif
 		fprintf(stderr, "\nError while changing to root directory!\n");
-		return EXIT_FAILURE;
+		return -1;
 	}
+#endif
+
+	/* Create work directory
+	 */
+	if (mkdir(config->work_directory, 0700) == -1) {
+		if (errno != EEXIST) {
+			fprintf(stderr, "Error creating work directory '%s'\n", config->work_directory);
+			return -1;
+		} else if (chmod(config->work_directory, 0700) == -1) {
+			fprintf(stderr, "Can't change access permissions of work directory '%s'\n", config->work_directory);
+			return -1;
+		}
+	}
+	if ((getuid() == 0) || (geteuid() == 0)) {
+		if (chown(config->work_directory, config->server_uid, config->server_gid) == -1) {
+			perror("chown(WorkDirectory)");
+			return -1;
+		}
+	}
+
+	/* Create the upload directory for PUT requests
+	 */
+	if (mkdir(config->upload_directory, 0733) == -1) {
+		if (errno != EEXIST) {
+			fprintf(stderr, "Error while creating UploadDirectory '%s'\n", config->upload_directory);
+			return -1;
+		}
+	}
+
+	if (stat(config->upload_directory, &status) == -1) {
+		perror("stat(UploadDirectory)");
+		return -1;
+	}
+	access_rights = 01733;
+	if (status.st_uid != 0) {
+		if ((getuid() == 0) || (geteuid() == 0)) {
+			if (chown(config->upload_directory, 0, 0) == -1) {
+				perror("chown(UploadDirectory, 0, 0)");
+				return -1;
+			}
+		} else {
+			access_rights = 01333;
+		}
+	}
+
+	if ((status.st_mode & 07777) != access_rights) {
+		if (chmod(config->upload_directory, access_rights) == -1) {
+			fprintf(stderr, "Can't change access permissions of UploadDirectory '%s'.\n", config->upload_directory);
+			return -1;
+		}
+	}
+
+#ifdef HAVE_MONITOR
+	/* Create monitor cache directory
+	 */
+	if (mkdir(config->monitor_directory, 0700) == -1) {
+		if (errno != EEXIST) {
+			fprintf(stderr, "Error creating monitor directory '%s'\n", config->monitor_directory);
+			return -1;
+		} else if (chmod(config->monitor_directory, 0700) == -1) {
+			fprintf(stderr, "Can't change access permissions of monitor directory '%s'\n", config->monitor_directory);
+			return -1;
+		}
+	}
+	if ((getuid() == 0) || (geteuid() == 0)) {
+		if (chown(config->monitor_directory, config->server_uid, config->server_gid) == -1) {
+			perror("chown(MonitorDirectory)");
+			return -1;
+		}
+	}
+#endif
+
+	/* Create logfiles
+	 */
+#ifdef HAVE_CHROOT
+	touch_logfiles(config, config->server_root);
+#else
+	touch_logfiles(config, "/");
+#endif
 
 	/* Change userid
 	 */
@@ -1826,7 +1974,7 @@ int run_server(t_settings *settings) {
 			}
 		}
 		fprintf(stderr, "\nError while changing uid/gid!\n");
-		return EXIT_FAILURE;
+		return -1;
 	} while (false);
 
 	if (settings->daemon == false) {
@@ -1836,6 +1984,16 @@ int run_server(t_settings *settings) {
 		signal(SIGINT, SIG_IGN);
 	}
 
+	/* Set signal handlers
+	 */
+	if (config->wait_for_cgi == false) {
+		signal(SIGCHLD, SIG_IGN);
+	}
+	signal(SIGPIPE, SIG_IGN);
+	signal(SIGTSTP, SIG_IGN);
+	signal(SIGABRT, SIG_IGN);
+	signal(SIGQUIT, SIG_IGN);
+	signal(SIGSEGV, SEGV_handler);
 	signal(SIGTERM, TERM_handler);
 	signal(SIGHUP,  HUP_handler);
 	signal(SIGUSR1, USR1_handler);
@@ -1849,7 +2007,7 @@ int run_server(t_settings *settings) {
 	while (binding != NULL) {
 		if (listen(binding->socket, 16) == -1) {
 			perror("listen(http(s))");
-			return EXIT_FAILURE;
+			return -1;
 		}
 		binding = binding->next;
 	}
@@ -1858,7 +2016,7 @@ int run_server(t_settings *settings) {
 	while (binding != NULL) {
 		if (listen(binding->socket, 1) == -1) {
 			perror("listen(commandchannel)");
-			return EXIT_FAILURE;
+			return -1;
 		}
 		binding = binding->next;
 	}
@@ -1876,6 +2034,15 @@ int run_server(t_settings *settings) {
 #ifdef HAVE_XSLT
 	init_xslt_module();
 #endif
+#ifdef HAVE_MONITOR
+	if (config->monitor_enabled) {
+		if (init_monitor_module(config) == -1) {
+			fprintf(stderr, "Monitor initialization failure.\n");
+			return -1;
+		}
+		monitor_server_start();
+	}
+#endif
 
 	/* Redirecting I/O to /dev/null
 	 */
@@ -1903,16 +2070,16 @@ int run_server(t_settings *settings) {
 	 */
 	if (pthread_attr_init(&task_runner_attr) != 0) {
 		fprintf(stderr, "Task-runner pthread init error");
-		return EXIT_FAILURE;
+		return -1;
 	} else if (pthread_attr_setdetachstate(&task_runner_attr, PTHREAD_CREATE_DETACHED) != 0) {
 		fprintf(stderr, "Task-runner pthread set detach state error");
-		return EXIT_FAILURE;
+		return -1;
 	} else if (pthread_attr_setstacksize(&task_runner_attr, PTHREAD_STACK_SIZE) != 0) {
 		fprintf(stderr, "Task-runner pthread set stack size error");
-		return EXIT_FAILURE;
+		return -1;
 	} else if (pthread_create(&task_runner_thread, &task_runner_attr, (void*)task_runner, (void*)config) != 0) {
 		fprintf(stderr, "Task-runner pthread create error");
-		return EXIT_FAILURE;
+		return -1;
 	}
 	pthread_attr_destroy(&task_runner_attr);
 
@@ -2003,10 +2170,17 @@ int run_server(t_settings *settings) {
 	}
 #endif
 
-	log_string(config->system_logfile, "Hiawatha v"VERSION" stopped");
-	close_logfiles(config->first_host, true);
+#ifdef HAVE_MONITOR
+	if (config->monitor_enabled) {
+		monitor_server_stop();
+		shutdown_monitor_module(config);
+	}
+#endif
 
-	return EXIT_SUCCESS;
+	log_string(config->system_logfile, "Hiawatha v"VERSION" stopped");
+	close_logfiles(config->first_host, 0);
+
+	return 0;
 }
 
 void show_help(char *hiawatha) {
@@ -2058,5 +2232,9 @@ int main(int argc, char *argv[]) {
 
 	/* Run Hiawatha
 	 */
-	return run_server(&settings);
+	if (run_server(&settings) == -1) {
+		return EXIT_FAILURE;
+	}
+
+	return EXIT_SUCCESS;
 }
