@@ -170,7 +170,7 @@ int send_buffer(t_session *session, const char *buffer, int size) {
 
 /* Send a HTTP header to the client. Header is not closed by this function.
  */
-int send_header(t_session *session, int code) {
+int send_header(t_session *session) {
 	char ecode[5], timestr[TIMESTR_SIZE];
 	const char *emesg;
 	time_t t;
@@ -179,8 +179,8 @@ int send_header(t_session *session, int code) {
 	/* Send HTTP header.
 	 */
 	ecode[4] = '\0';
-	snprintf(ecode, 4, "%d", code);
-	if ((emesg = http_error(code)) == NULL) {
+	snprintf(ecode, 4, "%d", session->return_code);
+	if ((emesg = http_error(session->return_code)) == NULL) {
 		emesg = unknown_code;
 	}
 	
@@ -284,7 +284,7 @@ int send_header(t_session *session, int code) {
 
 	/* Expires
 	 */
-	if ((session->expires > -1) && (code == 200)) {
+	if ((session->expires > -1) && (session->return_code == 200)) {
 		if (time(&t) == -1) {
 			return -1;
 		}
@@ -371,23 +371,23 @@ int send_chunk(t_session *session, const char *chunk, int size) {
 
 /* Send a HTTP code to the client. Used in case of an error.
  */
-int send_code(t_session *session, int code) {
+int send_code(t_session *session) {
 	int default_port;
 	char ecode[5], len[10], port[10];
 	const char *emesg;
 
-	if (code == -1) {
-		code = 500;
+	if (session->return_code == -1) {
+		session->return_code = 500;
 	}
 
 	/* Send simple HTTP error message.
 	 */
 	session->mimetype = NULL;
-	if (send_header(session, code) == -1) {
+	if (send_header(session) == -1) {
 		return -1;
 	}
 
-	switch (code) {
+	switch (session->return_code) {
 		case 301:
 			if (send_buffer(session, hs_lctn, 10) == -1) {
 				return -1;
@@ -467,9 +467,9 @@ int send_code(t_session *session, int code) {
 	}
 
 	ecode[4] = '\0';
-	snprintf(ecode, 4, "%d", code);
+	snprintf(ecode, 4, "%d", session->return_code);
 
-	if ((emesg = http_error(code)) == NULL) {
+	if ((emesg = http_error(session->return_code)) == NULL) {
 		emesg = unknown_code;
 	}
 	len[9] = '\0';
