@@ -41,169 +41,173 @@ enum t_section { syntax_error = -1, none, binding, virtual_host, directory, fcgi
 
 bool including = false;
 t_keyvalue *variables = NULL;
+char *index_xslt = CONFIG_DIR"/index.xslt";
 
 static t_host *new_host(void) {
 	t_host *host;
 
-	if ((host = (t_host*)malloc(sizeof(t_host))) != NULL) {
-		host->website_root       = NULL;
-		host->website_root_len   = 0;
-		host->start_file         = "index.html";
-		host->error_401_handler  = NULL;
-		host->error_403_handler  = NULL;
-		host->error_404_handler  = NULL;
-		host->error_501_handler  = NULL;
-		host->error_503_handler  = NULL;
-		host->access_logfile     = LOG_DIR"/access.log";
-		host->access_fileptr     = NULL;
-		host->access_fp          = &(host->access_fileptr);
-		host->access_time        = 0;
-		host->error_logfile      = LOG_DIR"/error.log";
-		init_charlist(&(host->hostname));
-		host->user_websites      = false;
-		host->execute_cgi        = false;
-		host->time_for_cgi       = 5;
-		host->show_index         = false;
-#ifdef HAVE_XSLT
-		host->index_xslt         = NULL;
-#endif
-		host->use_gz_file        = false;
-		host->access_list        = NULL;
-		host->alter_list         = NULL;
-		host->alter_fmode        = 0640;
-		host->run_on_alter       = NULL;
-		host->login_message      = "Private page";
-		host->passwordfile       = NULL;
-		host->groupfile          = NULL;
-		host->index_style        = NULL;
-		host->deny_bot           = NULL;
-		init_charlist(&(host->required_binding));
-		init_charlist(&(host->required_group));
-		init_charlist(&(host->alter_group));
-#ifdef HAVE_TOOLKIT
-		init_charlist(&(host->toolkit_rules));
-#endif
-		host->wrap_cgi           = NULL;
-		init_groups(&(host->groups));
-		init_charlist(&(host->volatile_object));
-		init_charlist(&(host->image_referer));
-		host->imgref_replacement = NULL;
-		host->envir_str          = NULL;
-		host->alias              = NULL;
-#ifdef HAVE_SSL
-		host->require_ssl        = false;
-#endif
-#ifdef HAVE_XSLT
-		host->use_xslt           = false;
-#endif
-		host->prevent_sqli       = false;
-		host->prevent_xss        = false;
-		host->prevent_csrf       = false;
-		host->follow_symlinks    = false;
-		host->enable_path_info   = false;
-		host->trigger_on_cgi_status = true;
-		init_charlist(&(host->fast_cgi));
-		host->secure_url         = true;
-		host->deny_body          = NULL;
-
-		host->next               = NULL;
-	} else {
+	if ((host = (t_host*)malloc(sizeof(t_host))) == NULL) {
 		perror("new_host()");
 		exit(EXIT_FAILURE);
 	}
 
+	host->website_root       = NULL;
+	host->website_root_len   = 0;
+	host->start_file         = "index.html";
+	host->error_401_handler  = NULL;
+	host->error_403_handler  = NULL;
+	host->error_404_handler  = NULL;
+	host->error_501_handler  = NULL;
+	host->error_503_handler  = NULL;
+	host->access_logfile     = LOG_DIR"/access.log";
+	host->access_fileptr     = NULL;
+	host->access_fp          = &(host->access_fileptr);
+	host->access_time        = 0;
+	host->error_logfile      = LOG_DIR"/error.log";
+	init_charlist(&(host->hostname));
+	host->user_websites      = false;
+	host->execute_cgi        = false;
+	host->time_for_cgi       = 5;
+#ifdef HAVE_XSLT
+	host->show_index         = NULL;
+#endif
+	host->use_gz_file        = false;
+	host->access_list        = NULL;
+	host->alter_list         = NULL;
+	host->alter_fmode        = 0640;
+	host->run_on_alter       = NULL;
+	host->login_message      = "Private page";
+	host->passwordfile       = NULL;
+	host->groupfile          = NULL;
+	host->deny_bot           = NULL;
+	init_charlist(&(host->required_binding));
+	init_charlist(&(host->required_group));
+	init_charlist(&(host->alter_group));
+#ifdef HAVE_TOOLKIT
+	init_charlist(&(host->toolkit_rules));
+#endif
+	host->wrap_cgi           = NULL;
+	init_groups(&(host->groups));
+	init_charlist(&(host->volatile_object));
+	init_charlist(&(host->image_referer));
+	host->imgref_replacement = NULL;
+	host->envir_str          = NULL;
+	host->alias              = NULL;
+#ifdef HAVE_SSL
+	host->require_ssl        = false;
+#endif
+#ifdef HAVE_XSLT
+	host->use_xslt           = false;
+#endif
+	host->prevent_sqli       = false;
+	host->prevent_xss        = false;
+	host->prevent_csrf       = false;
+	host->follow_symlinks    = false;
+	host->enable_path_info   = false;
+	host->trigger_on_cgi_status = true;
+	init_charlist(&(host->fast_cgi));
+	host->secure_url         = true;
+	host->deny_body          = NULL;
+#ifdef HAVE_MONITOR
+	host->monitor_stats      = NULL;
+	host->monitor_requests   = false;
+	host->monitor_host       = false;
+#endif
+
+	host->next               = NULL;
+
 	return host;
 }
 
 static t_directory *new_directory(void) {
 	t_directory *directory;
 
-	if ((directory = (t_directory*)malloc(sizeof(t_directory))) != NULL) {
-		directory->path                = NULL;
-		directory->wrap_cgi            = NULL;
-		directory->start_file          = NULL;
-		directory->execute_cgiset      = false;
-		directory->show_index_set      = false;
-#ifdef HAVE_XSLT
-		directory->index_xslt          = NULL;
-#endif
-		directory->use_gz_file_set     = false;
-		directory->follow_symlinks_set = false;
-		directory->access_list         = NULL;
-		directory->alter_list          = NULL;
-		directory->alter_fmode         = 0;
-		init_groups(&(directory->groups));
-		directory->passwordfile        = NULL;
-		directory->groupfile           = NULL;
-		init_charlist(&(directory->required_group));
-		init_charlist(&(directory->image_referer));
-		init_charlist(&(directory->alter_group));
-		directory->imgref_replacement  = NULL;
-		directory->max_clients         = -1;
-		directory->nr_of_clients       = 0;
-		directory->upload_speed        = 0;
-		directory->session_speed       = 0;
-		directory->envir_str           = NULL;
-		directory->time_for_cgi        = -1;
-		directory->run_on_download     = NULL;
-		pthread_mutex_init(&(directory->client_mutex), NULL);
-
-		directory->next                = NULL;
-	} else {
+	if ((directory = (t_directory*)malloc(sizeof(t_directory))) == NULL) {
 		perror("new_directory()");
 		exit(EXIT_FAILURE);
 	}
 
+	directory->path                = NULL;
+	directory->wrap_cgi            = NULL;
+	directory->start_file          = NULL;
+	directory->execute_cgiset      = false;
+#ifdef HAVE_XSLT
+	directory->show_index          = NULL;
+	directory->show_index_set      = false;
+#endif
+	directory->use_gz_file_set     = false;
+	directory->follow_symlinks_set = false;
+	directory->access_list         = NULL;
+	directory->alter_list          = NULL;
+	directory->alter_fmode         = 0;
+	init_groups(&(directory->groups));
+	directory->passwordfile        = NULL;
+	directory->groupfile           = NULL;
+	init_charlist(&(directory->required_group));
+	init_charlist(&(directory->image_referer));
+	init_charlist(&(directory->alter_group));
+	directory->imgref_replacement  = NULL;
+	directory->max_clients         = -1;
+	directory->nr_of_clients       = 0;
+	directory->upload_speed        = 0;
+	directory->session_speed       = 0;
+	directory->envir_str           = NULL;
+	directory->time_for_cgi        = TIMER_OFF;
+	directory->run_on_download     = NULL;
+	pthread_mutex_init(&(directory->client_mutex), NULL);
+
+	directory->next                = NULL;
+
 	return directory;
 }
 
 static t_fcgi_server *new_fcgi_server(void) {
 	t_fcgi_server *fcgi_server;
 
-	if ((fcgi_server = (t_fcgi_server*)malloc(sizeof(t_fcgi_server))) != NULL) {
-		fcgi_server->fcgi_id          = NULL;
-		fcgi_server->connect_to       = NULL;
-		fcgi_server->session_timeout  = 900;
-		fcgi_server->chroot           = NULL;
-		fcgi_server->chroot_len       = 0;
-		init_charlist(&(fcgi_server->extension));
-	} else {
+	if ((fcgi_server = (t_fcgi_server*)malloc(sizeof(t_fcgi_server))) == NULL) {
 		perror("new_fcgi_server()");
 		exit(EXIT_FAILURE);
 	}
 
+	fcgi_server->fcgi_id          = NULL;
+	fcgi_server->connect_to       = NULL;
+	fcgi_server->session_timeout  = 900;
+	fcgi_server->chroot           = NULL;
+	fcgi_server->chroot_len       = 0;
+	init_charlist(&(fcgi_server->extension));
+
 	return fcgi_server;
 }
 
 static t_binding *new_binding(void) {
 	t_binding *binding;
 
-	if ((binding = (t_binding*)malloc(sizeof(t_binding))) != NULL) {
-		binding->port                 = -1;
-		default_ipv4(&(binding->interface));
-#ifdef HAVE_SSL
-		binding->use_ssl              = false;
-		binding->ssl_key_cert         = NULL;
-		binding->required_ca          = NULL;
-		binding->verify_depth         = 1;
-		binding->ssl_context          = NULL;
-#endif
-		binding->binding_id           = NULL;
-		binding->socket               = -1;
-		binding->enable_trace         = false;
-		binding->enable_alter         = false;
-		binding->max_keepalive        = 50;
-		binding->max_request_size     = 64 * KILOBYTE;
-		binding->max_upload_size      = MEGABYTE;
-		binding->time_for_1st_request = 5;
-		binding->time_for_request     = 30;
-
-		binding->next                 = NULL;
-	} else {
+	if ((binding = (t_binding*)malloc(sizeof(t_binding))) == NULL) {
 		perror("new_binding()");
 		exit(EXIT_FAILURE);
 	}
 
+	binding->port                 = -1;
+	default_ipv4(&(binding->interface));
+#ifdef HAVE_SSL
+	binding->use_ssl              = false;
+	binding->ssl_key_cert         = NULL;
+	binding->required_ca          = NULL;
+	binding->verify_depth         = 1;
+	binding->ssl_context          = NULL;
+#endif
+	binding->binding_id           = NULL;
+	binding->socket               = -1;
+	binding->enable_trace         = false;
+	binding->enable_alter         = false;
+	binding->max_keepalive        = 50;
+	binding->max_request_size     = 64 * KILOBYTE;
+	binding->max_upload_size      = MEGABYTE;
+	binding->time_for_1st_request = 5;
+	binding->time_for_request     = 30;
+
+	binding->next                 = NULL;
+
 	return binding;
 }
 
@@ -211,16 +215,16 @@ static t_binding *new_binding(void) {
 t_url_toolkit *new_url_toolkit(void) {
 	t_url_toolkit *url_toolkit;
 
-	if ((url_toolkit = (t_url_toolkit*)malloc(sizeof(t_url_toolkit))) != NULL) {
-		url_toolkit->toolkit_id = NULL;
-		url_toolkit->toolkit_rule = NULL;
-
-		url_toolkit->next = NULL;
-	} else {
+	if ((url_toolkit = (t_url_toolkit*)malloc(sizeof(t_url_toolkit))) == NULL) {
 		perror("new_url_toolkit()");
 		exit(EXIT_FAILURE);
 	}
 
+	url_toolkit->toolkit_id = NULL;
+	url_toolkit->toolkit_rule = NULL;
+
+	url_toolkit->next = NULL;
+
 	return url_toolkit;
 }
 #endif
@@ -228,88 +232,96 @@ t_url_toolkit *new_url_toolkit(void) {
 t_config *default_config(void) {
 	t_config *config;
 
-	if ((config = (t_config*)malloc(sizeof(t_config))) != NULL) {
-		config->mimetype_config    = "mimetype.conf";
-
-		config->binding            = NULL;
-#ifdef HAVE_COMMAND
-		config->command_port       = NULL;
-#endif
-
-		config->server_root        = NULL;
-		config->server_uid         = (uid_t)ID_NOBODY;
-		config->server_gid         = (gid_t)ID_NOBODY;
-		config->server_string      = "Hiawatha v"VERSION;
-		init_groups(&(config->groups));
-#ifdef HAVE_SSL
-		config->dh_file            = NULL;
-		config->allowed_ciphers    = NULL;
-#endif
-		init_charlist(&(config->cgi_extension));
-		config->no_extension_as    = NULL;
-		config->total_connections  = 100;
-		config->connections_per_ip = 10;
-		config->socket_send_timeout = 3;
-		config->kill_timedout_cgi  = true;
-		config->wait_for_cgi       = true;
-		config->first_host         = new_host();
-		config->mimetype           = NULL;
-		config->directory          = NULL;
-		config->throttle           = NULL;
-		config->cgi_handler        = NULL;
-		config->cgi_wrapper        = NULL;
-		config->wrap_user_cgi      = false;
-		config->log_format         = hiawatha;
-		config->user_directory     = "public_html";
-		config->user_directory_set = false;
-		config->hide_proxy         = NULL;
-
-		config->pidfile            = PIDFILE_DIR"/hiawatha.pid";
-		config->system_logfile     = LOG_DIR"/system.log";
-		config->garbage_logfile    = NULL;
-		config->exploit_logfile    = LOG_DIR"/exploit.log";
-		config->logfile_mask       = NULL;
-
-		config->ban_on_denied_body = 0;
-		config->ban_on_garbage     = 0;
-		config->ban_on_max_per_ip  = 2;
-		config->ban_on_flooding    = 0;
-		config->ban_on_max_request_size = 0;
-		config->ban_on_sqli        = 0;
-		config->ban_on_timeout     = 0;
-		config->ban_on_wrong_password = 0;
-		config->kick_on_ban        = false;
-		config->reban_during_ban   = false;
-		config->max_wrong_passwords = 0;
-		config->flooding_count     = 0;
-		config->flooding_time      = 0;
-		config->reconnect_delay    = 0;
-		config->banlist_mask       = NULL;
-		config->fcgi_server        = NULL;
-		config->upload_directory   = UPLOAD_DIR;
-		config->upload_directory_len = strlen(UPLOAD_DIR);
-		config->create_upload_dir  = false;
-#ifdef HAVE_TOOLKIT
-		config->url_toolkit        = NULL;
-#endif
-#ifdef CYGWIN
-		config->platform           = windows;
-#endif
-
-#ifdef HAVE_CACHE
-		config->cache_size         = 10 * MEGABYTE;
-		config->cache_max_filesize = 256 * KILOBYTE;
-		config->cache_min_filesize = 1;
-#endif
-
-#ifdef HAVE_COMMAND
-		config->command_port       = NULL;
-#endif
-	} else {
+	if ((config = (t_config*)malloc(sizeof(t_config))) == NULL) {
 		perror("default_config()");
 		exit(EXIT_FAILURE);
 	}
 
+	config->mimetype_config    = "mimetype.conf";
+
+	config->binding            = NULL;
+#ifdef HAVE_COMMAND
+	config->command_port       = NULL;
+#endif
+
+#ifdef HAVE_CHROOT
+	config->server_root        = NULL;
+#endif
+	config->server_uid         = (uid_t)ID_NOBODY;
+	config->server_gid         = (gid_t)ID_NOBODY;
+	config->server_string      = "Hiawatha v"VERSION;
+	init_groups(&(config->groups));
+#ifdef HAVE_SSL
+	config->dh_file            = NULL;
+	config->allowed_ciphers    = NULL;
+#endif
+	init_charlist(&(config->cgi_extension));
+	config->no_extension_as    = NULL;
+	config->total_connections  = 100;
+	config->connections_per_ip = 10;
+	config->socket_send_timeout = 3;
+	config->kill_timedout_cgi  = true;
+	config->wait_for_cgi       = true;
+	config->first_host         = new_host();
+	config->mimetype           = NULL;
+	config->directory          = NULL;
+	config->throttle           = NULL;
+	config->cgi_handler        = NULL;
+	config->cgi_wrapper        = NULL;
+	config->wrap_user_cgi      = false;
+	config->log_format         = hiawatha;
+	config->user_directory     = "public_html";
+	config->user_directory_set = false;
+	config->hide_proxy         = NULL;
+
+	config->pidfile            = PIDFILE_DIR"/hiawatha.pid";
+	config->system_logfile     = LOG_DIR"/system.log";
+	config->garbage_logfile    = NULL;
+	config->exploit_logfile    = LOG_DIR"/exploit.log";
+	config->logfile_mask       = NULL;
+
+	config->ban_on_denied_body = 0;
+	config->ban_on_garbage     = 0;
+	config->ban_on_max_per_ip  = 2;
+	config->ban_on_flooding    = 0;
+	config->ban_on_max_request_size = 0;
+	config->ban_on_sqli        = 0;
+	config->ban_on_timeout     = 0;
+	config->ban_on_wrong_password = 0;
+	config->kick_on_ban        = false;
+	config->reban_during_ban   = false;
+	config->max_wrong_passwords = 0;
+	config->flooding_count     = 0;
+	config->flooding_time      = 0;
+	config->reconnect_delay    = 0;
+	config->banlist_mask       = NULL;
+	config->fcgi_server        = NULL;
+	config->work_directory     = WORK_DIR;
+	config->upload_directory   = NULL;
+	config->upload_directory_len = 0;
+#ifdef HAVE_TOOLKIT
+	config->url_toolkit        = NULL;
+#endif
+#ifdef CYGWIN
+	config->platform           = windows;
+#endif
+
+#ifdef HAVE_CACHE
+	config->cache_size         = 10 * MEGABYTE;
+	config->cache_max_filesize = 256 * KILOBYTE;
+	config->cache_min_filesize = 1;
+#endif
+
+#ifdef HAVE_COMMAND
+	config->command_port       = NULL;
+#endif
+
+#ifdef HAVE_MONITOR
+	config->monitor_enabled    = false;
+	config->monitor_directory  = WORK_DIR"/monitor";
+	config->monitor_stats_interval = 3600;
+#endif
+
 	return config;
 }
 
@@ -523,15 +535,7 @@ int check_configuration(t_config *config) {
 	t_host *host;
 	char c;
 	int i;
-	t_binding *binding;
-
-	binding = config->binding;
-	while (binding != NULL) {
-		if (binding->enable_alter) {
-			config->create_upload_dir = true;
-		}
-		binding = binding->next;
-	}
+	size_t len;
 
 	if (config->first_host->hostname.size == 0) {
 		fprintf(stderr, "The default website has no hostname.\n");
@@ -543,6 +547,31 @@ int check_configuration(t_config *config) {
 		return -1;
 	}
 
+	len = strlen(config->work_directory);
+
+	config->upload_directory_len = len + 7;
+	if ((config->upload_directory = (char*)malloc(config->upload_directory_len + 1)) == NULL) {
+		return -1;
+	}
+	memcpy(config->upload_directory, config->work_directory, len);
+	strcpy(config->upload_directory + len, "/upload");
+
+#ifdef HAVE_MONITOR
+	if ((config->monitor_directory = (char*)malloc(len + 9)) == NULL) {
+		return -1;
+	}
+	memcpy(config->monitor_directory, config->work_directory, len);
+	strcpy(config->monitor_directory + len, "/monitor");
+
+	if ((config->monitor_enabled) && (config->first_host->next != NULL)) {
+		host = config->first_host->next;
+		if (strcmp(host->hostname.item[0], "monitor") == 0) {
+			host->website_root = config->monitor_directory;
+			host->website_root_len = strlen(host->website_root);
+		}
+	}
+#endif
+
 	host = config->first_host;
 	while (host != NULL) {
 		for (i = 0; i < host->hostname.size; i++) {
@@ -557,10 +586,6 @@ int check_configuration(t_config *config) {
 			return -1;
 		}
 
-		if (host->show_index && (host->fast_cgi.size > 0)) {
-			fprintf(stderr, "Warning: both FastCGI and ShowIndex are enabled for host %s. This can cause undesired behaviour.\n", *(host->hostname.item));
-		}
-
 		host = host->next;
 	}
 
@@ -634,6 +659,10 @@ static bool system_setting(char *key, char *value, t_config *config) {
 	t_binding *binding;
 	char *port, *password;
 #endif
+#ifdef HAVE_MONITOR
+	t_host *monitor_host;
+	char *alist;
+#endif
 
 #ifdef HAVE_SSL
 	if (strcmp(key, "allowedciphers") == 0) {
@@ -819,6 +848,54 @@ static bool system_setting(char *key, char *value, t_config *config) {
 		if ((config->mimetype_config = strdup(value)) != NULL) {
 			return true;
 		}
+#ifdef HAVE_MONITOR
+	} else if (strcmp(key, "monitorserver") == 0) {
+		monitor_host = new_host();
+		monitor_host->next = config->first_host->next;
+		config->first_host->next = monitor_host;
+
+		if (parse_charlist("monitor", &(monitor_host->hostname)) == -1) {
+			return false;
+		}
+
+		monitor_host->website_root = config->monitor_directory;
+		monitor_host->website_root_len = strlen(monitor_host->website_root);
+
+		if ((monitor_host->access_logfile = strdup(LOG_DIR"/monitor-access.log")) == NULL) {
+			return false;
+		} else if ((monitor_host->error_logfile = strdup(LOG_DIR"/monitor-error.log")) == NULL) {
+			return false;
+		}
+
+
+		rest = "allow %s, deny all";
+		if ((alist = (char*)malloc(strlen(rest) + strlen(value) + 1)) == NULL) {
+			return false;
+		}
+		sprintf(alist, rest, value);
+		if ((monitor_host->access_list = parse_accesslist(alist, false, NULL)) == NULL) {
+			return false;
+		}
+		free(alist);
+
+#ifdef HAVE_XSLT
+		if ((monitor_host->show_index = strdup("xml")) == false) {
+			return false;
+		}
+#endif
+
+		monitor_host->monitor_host = true;
+		config->monitor_enabled = true;
+
+		return true;
+	} else if (strcmp(key, "monitorstatsinterval") == 0) {
+		if ((config->monitor_stats_interval = str2int(value)) > 0) {
+			if (config->monitor_stats_interval < 360) {
+				config->monitor_stats_interval *= MINUTE;
+				return true;
+			}
+		}
+#endif
 	} else if (strcmp(key, "noextensionas") == 0) {
 		if ((config->no_extension_as = strdup(value)) != NULL) {
 			return true;
@@ -856,12 +933,14 @@ static bool system_setting(char *key, char *value, t_config *config) {
 				}
 			}
 		}
+#ifdef HAVE_CHROOT
 	} else if (strcmp(key, "serverroot") == 0) {
 		if (valid_directory(value)) {
 			if ((config->server_root = strdup(value)) != NULL) {
 				return true;
 			}
 		}
+#endif
 	} else if (strcmp(key, "serverstring") == 0) {
 		if ((strcmp(value, "none") == 0) || (strcmp(value, "-") == 0)) {
 			config->server_string = NULL;
@@ -907,13 +986,6 @@ static bool system_setting(char *key, char *value, t_config *config) {
 				}
 			}
 		}
-	} else if (strcmp(key, "uploaddirectory") == 0) {
-		if (valid_directory(value)) {
-			if ((config->upload_directory = strdup(value)) != NULL) {
-				config->upload_directory_len = strlen(config->upload_directory);
-				return true;
-			}
-		}
 	} else if (strcmp(key, "userdirectory") == 0) {
 		if ((*value != '/') && (strchr(value, '.') == NULL)) {
 			if ((config->user_directory = strdup(value)) != NULL) {
@@ -925,6 +997,12 @@ static bool system_setting(char *key, char *value, t_config *config) {
 		if (parse_yesno(value, &(config->wait_for_cgi)) == 0) {
 			return true;
 		}
+	} else if (strcmp(key, "workdirectory") == 0) {
+		if (valid_directory(value)) {
+			if ((config->work_directory = strdup(value)) != NULL) {
+				return true;
+			}
+		}
 	} else if (strcmp(key, "wrapusercgi") == 0) {
 		if (parse_yesno(value, &(config->wrap_user_cgi)) == 0) {
 			return true;
@@ -986,15 +1064,6 @@ static bool user_setting(char *key, char *value, t_host *host, t_tempdata **temp
 				}
 			}
 		}
-	} else if (strcmp(key, "indexstyle") == 0) {
-		if ((host->index_style = strdup(value)) != NULL) {
-			if (register_tempdata(tempdata, host->index_style, tc_data) != -1) {
-				return true;
-			} else {
-				free(host->index_style);
-				host->index_style = NULL;
-			}
-		}
 	} else if (strcmp(key, "loginmessage") == 0) {
 		if (strlen(value) < 64) {
 			if ((host->login_message = strdup(value)) != NULL) {
@@ -1053,17 +1122,20 @@ static bool user_setting(char *key, char *value, t_host *host, t_tempdata **temp
 				free(kv);
 			}
 		}
-	} else if (strcmp(key, "showindex") == 0) {
-		if (parse_yesno(value, &(host->show_index)) == 0) {
-			return true;
 #ifdef HAVE_XSLT
-		} else if (*value == '/') {
-			if ((host->index_xslt = strdup(value)) != NULL) {
-				host->show_index = true;
+	} else if (strcmp(key, "showindex") == 0) {
+		if (strcmp(value, "yes") == 0) {
+			host->show_index = index_xslt;
+			return true;
+		} else if (strcmp(value, "no") == 0) {	
+			host->show_index = NULL;
+			return true;
+		} else if ((*value == '/') || (strcmp(value, "xml") == 0)) {
+			if ((host->show_index = strdup(value)) != NULL) {
 				return true;
 			}
-#endif
 		}
+#endif
 	} else if (strcmp(key, "startfile") == 0) {
 		if (valid_start_file(value)) {
 			if ((value = strdup(value)) != NULL) {
@@ -1154,7 +1226,7 @@ static bool host_setting(char *key, char *value, t_host *host) {
 			return true;
 		}
 	} else if (strcmp(key, "hostname") == 0) {
-		if (parse_charlist(value, &(host->hostname)) == 0) {
+		if (parse_charlist(strlower(value), &(host->hostname)) == 0) {
 			return true;
 		}
 	} else if (strcmp(key, "imagereferer") == 0) {
@@ -1165,6 +1237,12 @@ static bool host_setting(char *key, char *value, t_host *host) {
 				}
 			}
 		}
+#ifdef HAVE_MONITOR
+	} else if (strcmp(key, "monitorrequests") == 0) {
+		if (parse_yesno(value, &(host->monitor_requests)) == 0) {
+			return true;
+		}
+#endif
 	} else if (strcmp(key, "preventcsrf") == 0) {
 		if (parse_yesno(value, &(host->prevent_csrf)) == 0) {
 			return true;
@@ -1321,19 +1399,23 @@ static bool directory_setting(char *key, char *value, t_directory *directory) {
 		if (parse_keyvaluelist(value, &(directory->envir_str), "=") != -1) {
 			return true;
 		}
+#ifdef HAVE_XSLT
 	} else if (strcmp(key, "showindex") == 0) {
-		if (parse_yesno(value, &(directory->show_index)) == 0) {
+		if (strcmp(value, "yes") == 0) {
+			directory->show_index = index_xslt;
 			directory->show_index_set = true;
 			return true;
-#ifdef HAVE_XSLT
-		} else if (*value == '/') {
-			if ((directory->index_xslt = strdup(value)) != NULL) {
-				directory->show_index = true;
+		} else if (strcmp(value, "no") == 0) {	
+			directory->show_index = NULL;
+			directory->show_index_set = true;
+			return true;
+		} else if ((*value == '/') || (strcmp(value, "xml") == 0)) {
+			if ((directory->show_index = strdup(value)) != NULL) {
 				directory->show_index_set = true;
 				return true;
 			}
-#endif
 		}
+#endif
 	} else if (strcmp(key, "startfile") == 0) {
 		if (valid_start_file(value)) {
 			if ((directory->start_file = strdup(value)) != NULL) {
@@ -1882,52 +1964,64 @@ int read_user_configfile(char *configfile, t_host *host, t_tempdata **tempdata)
 
 t_host *get_hostrecord(t_host *host, char *hostname, t_binding *binding) {
 	char *dots;
-	int i, len, len_hostname;
+	size_t len, len_hostname;
+	int i;
 
-	if (hostname != NULL) {
-		if ((dots = strchr(hostname, ':')) != NULL) {
-			*dots = '\0';
-		}
+	if (hostname == NULL) {
+		return NULL;
+	}
 
-		if ((len_hostname = strlen(hostname)) > 0) {
-			/* Hostname ends with a dot */
-			if (hostname[len_hostname - 1] == '.') {
-				len_hostname--;
+	if ((dots = strchr(hostname, ':')) != NULL) {
+		*dots = '\0';
+	}
+
+	if ((len_hostname = strlen(hostname)) == 0) {
+		return NULL;
+	}
+
+	/* Hostname ends with a dot
+	 */
+	if (hostname[len_hostname - 1] == '.') {
+		len_hostname--;
+		hostname[len_hostname] = '\0';
+	}
+
+	while (host != NULL) {
+		if (host->required_binding.size > 0) {
+			if (in_charlist(binding->binding_id, &(host->required_binding)) == false) {
+				/* Binding not allowed
+				 */
+				host = host->next;
+				continue;
 			}
 		}
 
-		while (host != NULL) {
-			if (host->required_binding.size != 0) {
-				if (in_charlist(binding->binding_id, &(host->required_binding)) == false) {
-					/* Binding not allowed */
-					host = host->next;
-					continue;
-				}
-			}
-
-			for (i = 0; i < host->hostname.size; i++) {
-				if (strncasecmp(hostname, *(host->hostname.item + i), len_hostname) == 0) {
-					/* Exact match */
+		for (i = 0; i < host->hostname.size; i++) {
+			if (strcmp(hostname, *(host->hostname.item + i)) == 0) {
+				/* Exact match
+				 */
+				return host;
+			} else if (strncmp(*(host->hostname.item + i), "*.", 2) == 0) {
+				/* Wildcard in configuration
+				 */
+				if (strcmp(hostname, *(host->hostname.item + i) + 2) == 0) {
+					/* Only domainname requested
+					 */
 					return host;
-				} else if (strncasecmp(*(host->hostname.item + i), "*.", 2) == 0) {
-					/* Wildcard in configuration */
-					if (strncmp(hostname, *(host->hostname.item + i) + 2, len_hostname) == 0) {
-						/* Only domainname requested */
-						return host;
-					} else {
-						len = strlen(*(host->hostname.item + i));
-						if (len_hostname >= len) {
-							if (strncasecmp(hostname + len_hostname - len + 1, *(host->hostname.item + i) + 1, len - 1) == 0) {
-								/* Wildcard match for hostname */
-								return host;
-							}
+				} else {
+					len = strlen(*(host->hostname.item + i));
+					if (len_hostname >= len) {
+						if (strcmp(hostname + len_hostname - len + 1, *(host->hostname.item + i) + 1) == 0) {
+							/* Wildcard match for hostname
+							 */
+							return host;
 						}
 					}
 				}
 			}
-
-			host = host->next;
 		}
+
+		host = host->next;
 	}
 
 	return NULL;
