@@ -22,6 +22,9 @@
 #include "liblist.h"
 #include "session.h"
 #include "log.h"
+#ifdef HAVE_MONITOR
+#include "monitor.h"
+#endif
 #ifdef HAVE_COMMAND
 #include "command.h"
 #endif
@@ -31,6 +34,7 @@ int new_client_id = 0;
 /* Set the entries in a session-record to the default values.
  */
 static void clear_session(t_session *session) {
+	session->time = time(NULL);
 	session->cgi_type = no_cgi;
 	session->cgi_handler = NULL;
 	session->fcgi_server = NULL;
@@ -104,9 +108,10 @@ void init_session(t_session *session) {
 	session->request = NULL;
 	session->buffer_size = 0;
 	session->bytes_in_buffer = 0;
-	session->flooding_timer = time(NULL);
 
 	clear_session(session);
+
+	session->flooding_timer = session->time;
 }
 
 /* Reset a session-record for reuse.
@@ -344,12 +349,11 @@ int copy_directory_settings(t_session *session) {
 				if (dir->execute_cgiset) {
 					session->host->execute_cgi = dir->execute_cgi;
 				}
+#ifdef HAVE_XSLT
 				if (dir->show_index_set) {
 					session->host->show_index = dir->show_index;
-#ifdef HAVE_XSLT
-					session->host->index_xslt = dir->index_xslt;
-#endif
 				}
+#endif
 				if (dir->follow_symlinks_set) {
 					session->host->follow_symlinks = dir->follow_symlinks;
 				}
@@ -385,7 +389,7 @@ int copy_directory_settings(t_session *session) {
 					session->host->alter_group.size = dir->alter_group.size;
 					session->host->alter_group.item = dir->alter_group.item;
 				}
-				if (dir->time_for_cgi > -1) {
+				if (dir->time_for_cgi > TIMER_OFF) {
 					session->host->time_for_cgi = dir->time_for_cgi;
 				}
 				break;
@@ -397,23 +401,6 @@ int copy_directory_settings(t_session *session) {
 	return 200;
 }
 
-/* Complete path in chrooted session?
- *  *  */
-char *file_in_chroot(t_session *session, char *file) {
-	size_t length;
-
-	if (session->config->server_root != NULL) {
-		length = strlen(session->config->server_root);
-		if (strlen(session->host->passwordfile) > length) {
-			if (memcmp(session->host->passwordfile, session->config->server_root, length) == 0) {
-				file += length;
-			}
-		}
-	}
-
-	return file;
-}
-
 /* Check if User-Agent string contains deny_bot substring.
  */
 bool client_is_rejected_bot(t_session *session) {
@@ -475,6 +462,12 @@ int prevent_xss(t_session *session) {
 		str++;
 	}
 
+	if (result > 0) {
+#ifdef HAVE_MONITOR
+		monitor_counter_exploit_attempt(session);
+#endif
+	}
+
 	return result;
 }
 
@@ -530,6 +523,9 @@ int prevent_sqli(t_session *session, char *str, int length, char **newstr) {
 		newlength = 0;
 	} else {
 		*(*newstr + newlength) = '\0';
+#ifdef HAVE_MONITOR
+		monitor_counter_exploit_attempt(session);
+#endif
 	}
 
 	return newlength;
@@ -581,6 +577,10 @@ int prevent_csrf(t_session *session) {
 		log_error(session, "CSRF attempt detected with no request body");
 	}
 
+#ifdef HAVE_MONITOR
+	monitor_counter_exploit_attempt(session);
+#endif
+
 	return 1;
 }
 
