@@ -30,6 +30,7 @@ typedef enum { missing_slash, require_ssl, location } t_cause_of_301;
 typedef struct type_session {
 	t_config        *config;
 
+	time_t          time;
 	int             client_id;
 	int             client_socket;
 	t_binding       *binding;
@@ -118,7 +119,6 @@ int  get_homedir(t_session *session, char *username);
 bool duplicate_host(t_session *session);
 bool is_volatile_object(t_session *session);
 int  copy_directory_settings(t_session *session);
-char *file_in_chroot(t_session *session, char *file);
 bool client_is_rejected_bot(t_session *session);
 int prevent_xss(t_session *session);
 int  prevent_sqli(t_session *session, char *str, int length, char **newstr);
