@@ -314,7 +314,7 @@ int send_file(t_session *session) {
 
 	do {
 		retval = -1;
-		if (send_header(session, session->return_code) == -1) {
+		if (send_header(session) == -1) {
 			break;
 		}
 		if (session->return_code == 401) {
@@ -666,7 +666,7 @@ int execute_cgi(t_session *session) {
 		return retval;
 	}
 
-	cgi_info.deadline = time(NULL) + session->host->time_for_cgi;
+	cgi_info.deadline = session->time + session->host->time_for_cgi;
 
 	do {
 		if (time(NULL) > cgi_info.deadline) {
@@ -762,7 +762,8 @@ int execute_cgi(t_session *session) {
 							}
 
 							if (strcasestr(cgi_info.input_buffer, "\nLocation:") != NULL) {
-								if (send_header(session, 302) == -1) {
+								session->return_code = 302;
+								if (send_header(session) == -1) {
 									retval = rs_DISCONNECT;
 									break;
 								}
@@ -790,14 +791,14 @@ int execute_cgi(t_session *session) {
 								}
 								session->return_code = retval;
 								if (retval == 200) {
-									if (send_header(session, retval) == -1) {
+									if (send_header(session) == -1) {
 										retval = rs_DISCONNECT;
 										break;
 									}
 								} else {
 									break;
 								}
-							} else if (send_header(session, session->return_code) == -1) {
+							} else if (send_header(session) == -1) {
 								retval = rs_DISCONNECT;
 								break;
 							}
@@ -911,282 +912,6 @@ int execute_cgi(t_session *session) {
 	return retval;
 }
 
-/* Read the content of a directory and send it in a formatted list to the client.
- */
-int show_index(t_session *session) {
-	int total_files = 0, retval = -1, len;
-	off_t total_fsize = 0;
-	char size[FILESIZE_BUFFER_SIZE], color = '0', timestr[33], *slash, *link;
-	struct tm s;
-	t_filelist *filelist = NULL, *file;
-	t_keyvalue *alias;
-	bool root_dir;
-
-#ifdef HAVE_COMMAND
-	increment_counter(COUNTER_INDEX);
-#endif
-
-	session->mimetype = NULL;
-
-	if ((slash = strrchr(session->file_on_disk, '/')) == NULL) {
-		return 500;
-	}
-	*(slash + 1) = '\0';
-
-	switch (is_directory(session->file_on_disk)) {
-		case error:
-		case no:
-			return 500;
-		case not_found:
-			return 404;
-		case no_access:
-			log_error(session, fb_filesystem);
-			return 403;
-			break;
-		case yes:
-			break;
-	}
-
-	if (session->host->follow_symlinks == false) {
-		switch (contains_not_allowed_symlink(session->file_on_disk, session->host->website_root)) {
-			case error:
-				log_error(session, "error while scanning file for symlinks");
-				return 500;
-			case not_found:
-				return 404;
-			case no_access:
-			case yes:
-				log_error(session, fb_symlink);
-				return 403;
-			case no:
-				break;
-		}
-	}
-
-    /* HTTP/1.0 has no knowledge about chunked Transfer-Encoding.
-	 */
-	if (*(session->http_version + 7) == '0') {
-		session->keep_alive = false;
-	}
-
-	if (send_header(session, 200) == -1) {
-		return -1;
-	}
-
-	if (session->request_method == HEAD) {
-		send_buffer(session, "\r\n", 2);
-		return 200;
-	}
-
-	/* Read directory content
-	 */
-	if ((filelist = read_filelist(session->file_on_disk)) == NULL) {
-		return 500;
-	}
-
-	/* Add aliasses to directory list
-	 */
-	if (strcmp(session->uri, "/") == 0) {
-		alias = session->host->alias;
-		while (alias != NULL) {
-			if ((file = (t_filelist*)malloc(sizeof(t_filelist))) == NULL) {
-				remove_filelist(filelist);
-				return 500;
-			} else if ((file->name = strdup(alias->key + 1)) == NULL) {
-				free(file);
-				remove_filelist(filelist);
-				return 500;
-			}
-			file->size = 0;
-			file->time = time(NULL);
-			file->is_dir = true;
-			file->next = filelist;
-			filelist = file;
-
-			alias = alias->next;
-		}
-	}
-
-	file = filelist = sort_filelist(filelist);
-
-#ifdef HAVE_XSLT
-	if (session->host->index_xslt != NULL) {
-		retval = show_index_via_xslt(session, filelist);
-		remove_filelist(filelist);
-		return retval;
-	}
-#endif
-
-	do {
-		if (session->keep_alive) {
-			if (send_buffer(session, hs_chunked, 28) == -1) {
-				break;
-			}
-		} 
-		if (send_buffer(session, "Content-Type: text/html\r\n\r\n", 27) == -1) {
-			break;
-		} else if (send_buffer(session, NULL, 0) == -1) {
-			break;
-		}
-		session->header_sent = true;
-
-		if (send_chunk(session, "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01//EN\" \"http://www.w3.org/TR/html4/strict.dtd\">\n", 91) == -1) {
-			break;
-		} else if (send_chunk(session, "<html>\n<head>\n<title>Index of ", 30) == -1) {
-			break;
-		} else if (send_chunk(session, session->uri, session->uri_len) == -1) {
-			break;
-		} else if (send_chunk(session,	"</title>\n", 9) == -1) {
-			break;
-		}
-
-		if (session->host->index_style != NULL) {
-			if (send_chunk(session, "<link rel=\"stylesheet\" type=\"text/css\" href=\"", 45) == -1) {
-				break;
-			} else if (send_chunk(session, session->host->index_style, strlen(session->host->index_style)) == -1) {
-				break;
-			} else if (send_chunk(session, "\">\n", 3) == -1) {
-				break;
-			}
-		} else {
-			if (send_chunk(session, "<style type=\"text/css\">body { margin:50px 100px 50px 100px ; background-color:#000008 ; font-family:sans-serif } table.list { width:100% ; border-left:5px solid #8080ff ; border-right:5px solid #8080ff ; border-collapse:collapse } a { color:#900000 ; text-decoration:none } a:visited { color:#404040 } a:hover { color:#ff4040 ; text-decoration:underline } div.path { color:#ffffff ; font-size:20px ; text-align:right ; font-weight:bold ; margin-bottom:10px ; margin-right:25px} tr { font-size:12px } th { background-color:#8080ff } th.left { width:40px } th.filename { text-align:left } th.date { width:150px } th.size { width:80px ; text-align:right} th.right { width:30px } td.left { padding-left:5px } td.size { text-align:right } tr.line0 { background-color:#e0e0ff } tr.line1 { background-color:#e8e8ff } tr.bottom { background-color:#8080ff ; font-weight:bold } td.totalsize { text-align:right }</style>\n", 910) == -1) {
-				break;
-			}
-		}
-
-		if (send_chunk(session,"</head>\n<body>\n<div class=\"path\">", 33) == -1) { 
-			break;
-		} else if (send_chunk(session, *(session->host->hostname.item), strlen(*(session->host->hostname.item))) == -1) {
-			break;
-		} else if (send_chunk(session, " : ", 3) == -1) {
-			break;
-		} else if (send_chunk(session, session->uri, session->uri_len) == -1) {
-			break;
-		} else if (send_chunk(session, "</div>\n<table class=\"list\">\n<tr class=\"header\"><th class=\"left\"></th><th class=\"filename\">Filename</th><th class=\"date\">Date</th><th class=\"size\">Size</th><th class=\"right\"></th></tr>\n", 184) == -1) {
-			break;
-		} 
-
-		root_dir = (strcmp(session->uri, "/") == 0);
-
-		while (file != NULL) {
-			if (file->is_dir && root_dir) {
-				if (strcmp(file->name, "..") == 0) {
-					file = file->next;
-					continue;
-				}
-			}
-
-			if (send_chunk(session, "<tr class=\"line", 15) == -1) {
-				break;
-			} else if (send_chunk(session, &color, 1) == -1) {
-				break;
-			} else if (send_chunk(session, "\"><td class=\"left\">", 19) == -1) {
-				break;
-			}
-			if (file->is_dir) {
-				if (send_chunk(session, "[dir]", 5) == -1) {
-					break;
-				}
-			}
-
-			if (send_chunk(session, "</td><td class=\"filename\"><a href=\"", 35) == -1) {
-				break;
-			}
-
-			len = strlen(file->name);
-
-			if (url_encode(file->name, &link) == -1) {
-				break;
-			}
-			if (link == NULL) {
-				if (send_chunk(session, file->name, len) == -1) {
-					break;
-				}
-			} else {
-				if (send_chunk(session, link, strlen(link)) == -1) {
-					free(link);
-					break;
-				}
-				free(link);
-			}
-
-			if (file->is_dir) {
-				if (send_chunk(session, "/", 1) == -1) {
-					break;
-				}
-			}
-
-			if (send_chunk(session, "\">", 2) == -1) {
-				break;
-			} else if (send_chunk(session, file->name, len) == -1) {
-				break;
-			}
-
-			if (file->is_dir) {
-				if (send_chunk(session, "/", 1) == -1) {
-					break;
-				}
-			}
-			if (send_chunk(session, "</a></td><td class=\"date\">", 26) == -1) {
-				break;
-			}
-			s = *localtime(&(file->time));
-			*(timestr + 32) = '\0';
-			strftime(timestr, 32, "%d %b %Y, %X", &s);
-			if (send_chunk(session, timestr, strlen(timestr)) == -1) {
-				break;
-			} else if (send_chunk(session, "</td><td class=\"size\">", 22) == -1) {
-				break;
-			}
-			if (file->is_dir == false) {
-				len = filesize2str(size, FILESIZE_BUFFER_SIZE, file->size);
-				if (send_chunk(session, size, len) == -1) {
-					break;
-				}
-			}
-			if (send_chunk(session, "</td><td class=\"right\"></td></tr>\n", 34) == -1) {
-				break;
-			}
-			color = '0' + (1 - (color - '0'));
-
-			total_files++;
-			if (file->is_dir == false) {
-				total_fsize += file->size;
-			}
-			file = file->next;
-		}
-		if (file != NULL) {
-			break;
-		}
-
-		if (send_chunk(session,	"<tr class=\"bottom\"><td></td><td class=\"totalfiles\">", 51) == -1) {
-			break;
-		}
-		snprintf(size, FILESIZE_BUFFER_SIZE - 1, "%d", total_files);
-		if (send_chunk(session, size, strlen(size)) == -1) {
-			break;
-		} else if (send_chunk(session, " files</td><td></td><td class=\"totalsize\">", 42) == -1) {
-			break;
-		}
-		len = filesize2str(size, FILESIZE_BUFFER_SIZE, total_fsize);
-		if (send_chunk(session, size, len) == -1) {
-			break;
-		} else if (send_chunk(session, "</td><td></td></tr>\n</table>\n</body>\n</html>\n", 45) == -1) {
-			break;
-		} else  if (session->keep_alive) {
-			if (send_chunk(session, NULL, 0) == -1) {
-				break;
-			}
-		}
-
-		retval = 200;
-	} while (false);
-
-	remove_filelist(filelist);
-
-	return retval;
-}
-
 /* Send the result of an OPTIONS request method to the client.
  */
 int handle_options_request(t_session *session) {
@@ -1223,7 +948,8 @@ int handle_options_request(t_session *session) {
 		}
 	}
 
-	if (send_header(session, code) == -1) {
+	session->return_code = code;
+	if (send_header(session) == -1) {
 		return -1;
 	} else if (send_buffer(session, hs_allow, 25) == -1) {
 		return -1;
@@ -1280,7 +1006,7 @@ int handle_trace_request(t_session *session) {
 		 */
 		if (snprintf(buffer, MAX_TRACE_HEADER, "%d\r\nContent-Type: message/http\r\n\r\n", body_size) < 0) {
 			break;
-		} else if (send_header(session, 200) == -1) {
+		} else if (send_header(session) == -1) {
 			break;
 		} else if (send_buffer(session, hs_conlen, 16) == -1) {
 			break;
