@@ -25,6 +25,9 @@
 #include "libstr.h"
 #include "send.h"
 #include "log.h"
+#ifdef HAVE_COMMAND
+#include "command.h"
+#endif
 
 #define VALUE_SIZE 16
 #define XSLT_INDEX "/index.xslt\0"
@@ -35,6 +38,8 @@
 #define XML_CHUNK_LEN 2048
 
 extern char *hs_conlen;
+extern char *fb_filesystem;
+extern char *fb_symlink;
 
 /* Translate spacial characters
  */
@@ -352,7 +357,7 @@ static int apply_xslt_sheet(t_session *session, xmlDocPtr data_xml) {
 	 */
 	style_xml = xmlReadFile(session->xslt_file, NULL, 0);
 	if (style_xml == NULL) {
-		log_file_error(session, session->xslt_file, "XSLT is invalid XML");
+		log_file_error(session, session->xslt_file, "XSLT file contains invalid XML");
 		return 500;
 	}
 	xslt = xsltParseStylesheetDoc(style_xml);
@@ -423,7 +428,7 @@ int transform_xml(t_session *session) {
 	xmlDocPtr data_xml;
 	int result;
 
-	if (send_header(session, 200) == -1) {
+	if (send_header(session) == -1) {
 		return 500;
 	}
 
@@ -458,13 +463,102 @@ int add_tag(char **buffer, int *size, int extra_size, int *len, char *tag, char
 
 /* Apply XSLT to directory index
  */
-int show_index_via_xslt(t_session *session, t_filelist *file) {
+int show_index(t_session *session) {
 	xmlDocPtr data_xml;
-	char *text_xml, fsize_str[30], timestr[33], value[VALUE_SIZE + 1], *extension, *link;
+	char *text_xml, fsize_str[30], timestr[33], value[VALUE_SIZE + 1], *extension, *link, *slash;
 	int text_size, text_max, fsize_len, result, handle;
 	off_t total_fsize = 0;
-	bool root_dir;
+	bool root_dir, show_xml;
 	struct tm s;
+	t_filelist *filelist = NULL, *file;
+	t_keyvalue *alias;
+
+#ifdef HAVE_COMMAND
+	increment_counter(COUNTER_INDEX);
+#endif
+
+	session->mimetype = NULL;
+
+	if ((slash = strrchr(session->file_on_disk, '/')) == NULL) {
+		return 500;
+	}
+	*(slash + 1) = '\0';
+
+	switch (is_directory(session->file_on_disk)) {
+		case error:
+		case no:
+			return 500;
+		case not_found:
+			return 404;
+		case no_access:
+			log_error(session, fb_filesystem);
+			return 403;
+			break;
+		case yes:
+			break;
+	}
+
+	if (session->host->follow_symlinks == false) {
+		switch (contains_not_allowed_symlink(session->file_on_disk, session->host->website_root)) {
+			case error:
+				log_error(session, "error while scanning file for symlinks");
+				return 500;
+			case not_found:
+				return 404;
+			case no_access:
+			case yes:
+				log_error(session, fb_symlink);
+				return 403;
+			case no:
+				break;
+		}
+	}
+
+    /* HTTP/1.0 has no knowledge about chunked Transfer-Encoding.
+	 */
+	if (*(session->http_version + 7) == '0') {
+		session->keep_alive = false;
+	}
+
+	if (send_header(session) == -1) {
+		return -1;
+	}
+
+	if (session->request_method == HEAD) {
+		send_buffer(session, "\r\n", 2);
+		return 200;
+	}
+
+	/* Read directory content
+	 */
+	if ((filelist = read_filelist(session->file_on_disk)) == NULL) {
+		return 500;
+	}
+
+	/* Add aliasses to directory list
+	 */
+	if (strcmp(session->uri, "/") == 0) {
+		alias = session->host->alias;
+		while (alias != NULL) {
+			if ((file = (t_filelist*)malloc(sizeof(t_filelist))) == NULL) {
+				remove_filelist(filelist);
+				return 500;
+			} else if ((file->name = strdup(alias->key + 1)) == NULL) {
+				free(file);
+				remove_filelist(filelist);
+				return 500;
+			}
+			file->size = 0;
+			file->time = session->time;
+			file->is_dir = true;
+			file->next = filelist;
+			filelist = file;
+
+			alias = alias->next;
+		}
+	}
+
+	file = filelist = sort_filelist(filelist);
 
 	root_dir = (strcmp(session->uri, "/") == 0);
 
@@ -620,13 +714,21 @@ int show_index_via_xslt(t_session *session, t_filelist *file) {
 		return -1;
 	}
 
+	if (strcmp(session->host->show_index, "xml") == 0) {
+		show_xml = true;
+	} else if ((handle = open(session->host->show_index, O_RDONLY)) == -1) {
+		show_xml = true;
+	} else {
+		show_xml = false;
+	}
+
 	/* Show listing
 	 */
-	if ((handle = open(session->host->index_xslt, O_RDONLY)) == -1) {
+	if (show_xml) {
 		if (errno == EACCES) {
-			log_file_error(session, session->host->index_xslt, "access denied");
+			log_file_error(session, session->host->show_index, "access denied");
 		} else {
-			log_file_error(session, session->host->index_xslt, "file not found");
+			log_file_error(session, session->host->show_index, "file not found");
 		}
 
 		value[VALUE_SIZE] = '\0';
@@ -651,7 +753,7 @@ int show_index_via_xslt(t_session *session, t_filelist *file) {
 
 	close(handle);
 
-	session->xslt_file = session->host->index_xslt;
+	session->xslt_file = session->host->show_index;
 	data_xml = xmlReadMemory(text_xml, text_size, "index.xml", NULL, 0);
 
 	result = apply_xslt_sheet(session, data_xml);
